-- S E E Q S O R T . E X   -- Quicksort algorithm visual demonstration
-- based on a program written in QuickBASIC by Ethan Winer
----

include graphics.e

constant max_elements = 23,      -- the size of the text array
		delay = 0.8,     -- pause delay, change to suit
		   fg = 7,       -- foreground color
		   bg = 1,       -- background color
		   hi = 15 + 16  -- hi intensity flashing

sequence array, qstack
atom temp, start, tmp
integer I, J, stack_pointer, First, Last


procedure pause()
    start = time()
    while time() < start + delay do
    end while
    if get_key() = 27 then   -- Press escape (ascii-27) to end early
	abort(0)
    end if
end procedure

procedure update_screen()
    cursor(NO_CURSOR)
    text_color(fg) bk_color(bg)
    position(1, 63)
    puts(1, repeat(' ', 15))
    for X = 1 to max_elements do
	position(X, 24)
	if X = floor((Last + First)/2) then
	    text_color(bg) bk_color(fg)
	    puts(1, " Midpoint ==> \n")
	    text_color(fg) bk_color(bg)
	else
	    puts(1, repeat(' ', 14))
	end if
	position(X, 1)
	if X = First then
	    text_color(bg) bk_color(fg)
	    puts(1, " First ==> \n")
	    text_color(fg) bk_color(bg)
	else
	    puts(1, repeat(' ', 11)  )
	end if
    
	position(X, 13)
	if X = Last then
	    text_color(bg) bk_color(fg)
	    puts(1, " Last ==> \n")
	    text_color(fg) bk_color(bg)
	else
	    puts(1, repeat(' ', 11))
	end if
     
	position(X, 39)
	printf(1, "%6.2f", array[X])
	puts(1, repeat(' ', 18))
	text_color(bg) bk_color(fg)
	position(X, 47)
     
	if X = I then
	    puts(1, " <== I \n") 
	end if
	if X = J then
	    position(X, 55)
	    puts(1, " <== J \n")
	end if
    
	text_color(fg) bk_color(bg)
    end for
end procedure

procedure seeqsort()
    qstack = repeat(0, 12) -- create a stack big enough for the example.
    First = 1
    Last = 23
    while 1 do
	while 1 do
	    temp = array[floor((Last+First)/2)] -- seek midpoint
	    I = First
	    J = Last
      
	    while 1 do -- reverse both < and > below to sort descending
		while array[I] < temp do -- <, >, numbers only
		    I += 1
		    update_screen()
		    pause()
		end while
		while array[J] > temp do
		    J -= 1
		    update_screen()
		    pause()
		end while
		if I > J  then exit end if
		if I < J then
		    update_screen() --!
		    position(1, 63)
		    text_color(bg) bk_color(fg)
		    puts(1, " About to swap ")
		    text_color(hi) bk_color(bg)
		    position(I, 39)
		    printf(1, "%6.2f", array[I])
		    position(J, 39)
		    printf(1, "%6.2f", array[J])
		    text_color(fg) bk_color(bg)
		    pause()
	  
		    -- Swap
		    tmp=array[I] array[I]=array[J] array[J]=tmp
		    update_screen()
		    position(1, 63)
		    text_color(bg) bk_color(fg)
		    puts(1, "    Swapped    ")
		    pause()
		end if
		I += 1
		J -= 1
		if I > J then
		    exit
		end if
	    end while
	    if I < Last then              -- Done
		position(1, 63)
		text_color(bg) bk_color(fg)
		puts(1, " About to push ")
		pause()
		qstack[stack_pointer + 1] = I     -- push I
		qstack[stack_pointer + 2] = Last  -- push Last
		stack_pointer += 2
		--update_screen()
		position(1, 63)
		text_color(bg) bk_color(fg)
		puts(1, "    Pushed     ")
		pause()
	    end if
	    Last = J
	    if First >= Last then
		exit
	    end if
	end while
	if stack_pointer = 0 then exit end if
	position(1,63)
	text_color(bg) bk_color(fg)
	puts(1, "  About to pop ")
	pause()
	stack_pointer -= 2
	First = qstack[stack_pointer + 1]  -- pop First
	Last = qstack[stack_pointer + 2]   -- pop last
	update_screen()
	position(1, 63)
	text_color(bg) bk_color(fg)
	puts(1, "    Popped     ")
	pause()
    end while  
end procedure

--
-- Program execution begins here, and from here
-- the above routines are called
--

    stack_pointer=0

    array = repeat(0,max_elements)  -- create an array
    for x = 1 to max_elements do    -- fill with random numbers
	array[x] = (rand(50001) - 1 )/100 -- between 0 and 500
    end for

    text_color(fg) bk_color(bg)
    clear_screen()
    position(25, 1)
    puts(1,"Press escape to end the program early.")
    cursor(NO_CURSOR)
    seeqsort()
    cursor(UNDERLINE_CURSOR)         
    text_color(7) bk_color(0)  -- set screen to normal before leaving

